-- create database for demo
create database CteExample;
GO

use CteExample;
GO

-- create table for demo (SQL 2016)
-- i love the new DIE statement in SQL 2016 :)
drop table if exists dbo.simpletree
-- in case you do not have SQL 2016 use below line instead:
-- if exists(select object_id('dbo.simpletree')) drop table dbo.simpletree


-- create table for demo
create table dbo.simpletree (
id int
,parentId int
,name nvarchar(10)
)

-- insert some data in the table
insert into dbo.simpletree (id, parentId, name)
values
	(1, null, 'Ditlev')
	,(2, null,'Claus')
	,(3, 1,'Jane')
	,(4, 2,'John')
	,(5, 3, 'Brian')

-- show data in table
select * from dbo.simpletree

-- anchor query
select
	id
	,parentId
	,name
from
	dbo.simpletree
where 1=1
	and parentId is null

-- full cte
with cte as (
	select 
		id
		,name
		,parentId
		,cast(N'' as nvarchar(10)) as parentName
	from 
		dbo.simpletree s
	where 1=1
		and s.parentId is null

	union all

	select
		s.id
		,s.name
		,s.parentId
		,c.name as parentName
	from
		dbo.simpletree s
		join cte c
			on s.parentId = c.id
	)

select
	c.id
	,c.name
	,c.parentId
	,c.parentName
from
	cte c;

-- rewritten recursive CTE
with T0 as (
select 
	id
	,name
	,parentId
	,cast(N'' as nvarchar(10)) as parentName
from 
	dbo.simpletree s
where 1=1
	and s.parentId is null)
, T1 as (
select
	s.id
	,s.name
	,s.parentId
	,T0.name as parentName
from
	dbo.simpletree s
	join T0
		on s.parentId = T0.id
)
, T2 as (
select
	s.id
	,s.name
	,s.parentId
	,T1.name as parentName
from
	dbo.simpletree s
	join T1
		on s.parentId = T1.id
)
, Tn as (
select
	s.id
	,s.name
	,s.parentId
	,T2.name as parentName
from
	dbo.simpletree s
	join T2
		on s.parentId = T2.id
)
select * from T0
union all 
select * from T1
union all
select * from T2
union all
select * from Tn;


-- recursive CTE on complex hierarchy
with ctecomplex as (
select
	c.dwId
	,c.parentDwId
	,c.Tekst
	,cast(N'' as nvarchar(75)) as parentText
from
	dbo.complextree c
where 1=1
	and parentDwId is null

union all

select
	c.dwId
	,c.parentDwId
	,c.Tekst
	,ct.Tekst as parentText
from
	dbo.complextree c
	join ctecomplex ct
		on ct.dwId = c.parentDwId
)

select
	c.dwId
	,c.parentDwId
	,c.Tekst
	,c.parentText
from
	ctecomplex c

-- create nonclustered index
create nonclustered index nci_complextree on dbo.complextree (parentDwId)